<?php

/**
 * @file
 * Rules hooks and callbacks for Signup Status Rules module.
 * We use the signup data type from Signup Rules module to get signup
 * statuses from the database instead of defining our own data type.
 */

/**
 * Implements hook_rules_event_info().
 *
 * @ingroup rules
 */
function signup_status_rules_rules_event_info() {
  $events = array();
  $events['signup_status_rules_event_status_insert'] = array(
    'label' => t('Signup status was created'),
    'module' => 'Signup Status',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
      'status' => array('type' => 'signup_status', 'label' => t('Signup status')),
    ),
    'help' => 'This event is triggered when the signup status of an individual signup has been created by the Signup Status module.',
  );
  $events['signup_status_rules_event_status_update'] = array(
    'label' => t('Signup status was updated'),
    'module' => 'Signup Status',
    'arguments' =>  array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
      'status' => array('type' => 'signup_status', 'label' => t('Updated signup status')),
      'old_status' => array('type' => 'signup_status', 'label' => t('Unchanged signup status')),
    ),
    'help' => 'This event is triggered when the signup status of an individual signup has been changed by the Signup Status module.',
  );
  $events['signup_status_rules_event_status_delete'] = array(
    'label' => t('Signup status was deleted'),
    'module' => 'Signup Status',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
      'status' => array('type' => 'signup_status', 'label' => t('Signup status')),
    ),
    'help' => 'This event is triggered when the signup status of an individual signup has been deleted by the Signup Status module.',
  );
  return $events;
}

/**
 * Implements hook_rules_condition_info().
 *
 * @ingroup rules
 */
function signup_status_rules_rules_condition_info() {
  $conditions = array();
  $conditions['signup_status_rules_condition_status_has_value'] = array(
    'label' => t('Signup status has value'),
    'module' => 'Signup Status',
    'arguments' => array(
      'status' => array('type' => 'signup_status', 'label' => t('Status')),
    ),
    'base' => 'signup_status_rules_condition_status_has_value',
    'help' => t('Evaluates to TRUE if the unchanged signup status has the selected value.'),
  );
  $conditions['signup_status_limit'] = array(
    'label' => t('Signup status total limit'),
    'module' => 'Signup Status',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Signup content')),
      'status' => array('type' => 'signup_status', 'label' => t('Status')),
    ),
    'base' => 'signup_status_rules_condition_signup_status_limit',
    'help' => 'Evaluates to TRUE if the signup total for the given status is greater, equal or less the assigned limit.',
  );
  return $conditions;
}

/**
 * Verifies that a status has a specific value.
 */
function signup_status_rules_condition_status_has_value($status, $settings) {
  if ($status == $settings['signup_status']) {
    return TRUE;
  }
  return FALSE;
}

/**
 * Verifies the status limit for the given signup.
 */
function signup_status_rules_condition_signup_status_limit($node, $status, $settings) {
  $limit = $node->signup_status_limit[$status->cid];
  switch ($settings['signup_status_limit_condition']) {
    case 'greater':
      return $limit['total'] > $limit['limit'];
    case 'equal':
      return $limit['total'] == $limit['limit'];
    case 'less':
      return $limit['total'] < $limit['limit'];
  }
}

/**
 * Implements hook_rules_action_info().
 *
 * @ingroup rules
 */
function signup_status_rules_rules_action_info() {
  $actions = array();
  $actions['set_status'] = array(
    'label' => t('Set signup status'),
    'module' => 'Signup Status',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'base' => 'signup_status_rules_action_set_status',
    'help' => 'This action alters the signup status of a signup.',
  );
  $actions['mail_to_signed_up_users'] = array(
    'label' => t('Send mail to signed up users'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'eval input' => array('subject', 'message'),
    'base' => 'signup_status_rules_action_mail_users',
    'help' => 'Sends a mail to all users signed up for content.',
  );
  $actions['load_status'] = array(
      'label' => t('Load signup status by id'),
      'module' => 'Signup Status',
      'arguments' => array(
        'nid' => array('type' => 'number', 'label' => t('Signup ID')),
      ),
      'new variables' => array(
        'signup_status_loaded' => array(
          'type' => 'signup_status',
          'label' => t('Loaded signup status'),
        ),
      ),
      'base' => 'signup_status_rules_action_load_status',
      'help' => 'This action loads a signup status.',
  );
  $actions['load_status_user'] = array(
    'label' => t('Load the first user signed up for a given status'),
    'module' => 'Signup Status',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Signup content')),
      'status' => array('type' => 'signup_status', 'label' => t('Status')),
    ),
    'new variables' => array(
      'signup_status_user_loaded' => array(
        'type' => 'user',
        'label' => t('Loaded signup status user'),
      ),
    ),
    'base' => 'signup_status_rules_action_load_status_user',
    'help' => 'This action loads the first user signed up for a given status.',
  );
  return $actions;
}

/**
 * Implements hook_rules_action_info_alter().
 *
 * Override Signup Rules mail action to allow mail to be set by status as well.
 */
function signup_status_rules_rules_action_info_alter(&$actions) {
  if (!empty($actions['mail_to_signup_users'])) {
    unset($actions['mail_to_signup_users']);
  }
}

/**
 * Action: Set signup status.
 */
function signup_status_rules_action_set_status($signup, $settings) {
  $updated = drupal_clone($signup);
  $updated->status = $settings['signup_status'];
  db_query('UPDATE {signup_log} SET status = %d WHERE sid = %d', $updated->status, $updated->sid);
  $updated->old_status = $signup->status;
  _signup_status_change('update', $updated);
}

/**
 * Action: Mail signup users.
 */
function signup_status_rules_action_mail_users($node, $settings) {
  $signups = array();

  // Build the proper query for selecting limited signups.
  $limit = $settings['limit'];

  // If 'all' is selected then ignore the other values.
  if (empty($limit['all'])) {
    foreach ($limit as $key => $value) {
      if ($value) {
        if ($key == 'attended') {
          $result = db_query('SELECT u.uid, u.name, u.mail, u.language, sl.* FROM {signup_log} sl INNER JOIN {users} u ON u.uid = sl.uid WHERE sl.nid = %d AND attended = 1', $node->nid);
        }
        elseif ($key == 'not attended') {
          $result = db_query('SELECT u.uid, u.name, u.mail, u.language, sl.* FROM {signup_log} sl INNER JOIN {users} u ON u.uid = sl.uid WHERE sl.nid = %d AND attended = 0', $node->nid);
        }
        elseif ($key != 'all') {
          // This is a signup status option.
          $result = db_query('SELECT u.uid, u.name, u.mail, u.language, sl.* FROM {signup_log} sl INNER JOIN {users} u ON u.uid = sl.uid WHERE sl.status = %d', $key);
        }
      }
      // Add the signups to the array.
      while ($signup = db_fetch_object($result)) {
        if (!in_array($signups, $signup)) {
          $signups[] = $signup;
        }
      }
    }
  }

  // Send the signup mail.
  module_load_include('inc', 'signup_rules', 'signup_rules.rules');
  signup_rules_send_mail($node, $signups, $settings);
}

/**
 * Loads the status object for the given status id.
 */
function signup_status_rules_action_load_status($sid) {
  return array('signup_status_loaded' => signup_status_rules_load_status($sid));
}

/**
 * Loads the first signed up user for a given status.
 */
function signup_status_rules_action_load_status_user($node, $status) {
  $query = "SELECT uid FROM {signup_log} s WHERE s.nid = %d AND s.status = %d ORDER BY s.sid ASC";
  $uid = db_result(db_query_range($query, $node->nid, $status->cid, 0, 1));
  return array('signup_status_user_loaded' => user_load($uid));
}

/**
 * Implements hook_rules_data_type_info().
 */
function signup_status_rules_rules_data_type_info() {
  return array(
    'signup_status' => array(
      'label' => t('Signup status'),
      'module' => 'signup_status',
      'identifiable' => TRUE,
      'class' => 'rules_data_type_signup_status',
    ),
  );
}

/**
 * Defines the signup data type.
 */
class rules_data_type_signup_status extends rules_data_type {

  function load($sid) {
    return signup_status_rules_load_status($sid);
  }

  function get_identifier() {
    $signup = $this->get();
    return $signup->status;
  }
}
